﻿' Copyright (c) 2007 Manfred Clemens erst.: 06.11.2007, letzte Änderung: 04.04.2010
' ---------------------------------------------------------------------------------
Option Explicit On              ' explizite Deklarationen aller Variablen
Option Strict On                ' strikte Datentypkonvertierungen

#Region " Imports "
Imports Microsoft.VisualBasic.Devices
Imports System.IO
#End Region

Public Class clsGlobal

#Region " Members variable declaration's "
  Private _AppPath As String
  Public Shared FormArray As New ArrayList        ' speichert Namen der Formulare
  Private Shared ci As New ComputerInfo           ' Speicherinformationen
  Public Shared nextPage2, nextPage3 As Boolean         ' Druckvariable
  Public Shared str0, str1, str2, str3, str4 As String  ' Druckvariable
#End Region

#Region " Properties "
#Region " ... AppPath "
  Public ReadOnly Property AppPath() As String
    ' ---------------------------------------------------------------------------------
    ' Das Anwendungsverzeichnis der ausführbaren EXE-Datei ermitteln.
    ' Aufruf z.B.: AppPath & "Filename.xml"
    ' ---------------------------------------------------------------------------------
    Get
      ' Bei dieser Variante wird autom. der abschließende Backslash hinzugefügt.
      _AppPath = System.AppDomain.CurrentDomain.BaseDirectory
      Return _AppPath
    End Get

    '1. Alternative:
    'Dim sAppPath As String = Application.StartupPath
    'ggf. abschließenden Backslash hinzufügen
    'If Not sAppPath.EndsWith("\") Then sAppPath = sAppPath & "\"
    'Return sAppPath

    '2. Alternative:
    'Return IO.Path.GetDirectoryName(Reflection.Assembly.GetExecutingAssembly.Location)
  End Property
#End Region
#Region " ... Speicherinformationen "
  Public Shared ReadOnly Property AvailablePhysicalMemory() As ULong
    Get
      Return CType((ci.AvailablePhysicalMemory / (1024 * 1024)), ULong)
    End Get
  End Property

  Public Shared ReadOnly Property TotalPhysicalMemory() As ULong
    Get
      Return CType((ci.TotalPhysicalMemory / (1024 * 1024)), ULong)
    End Get
  End Property
#End Region
#Region " ... Euro-Zeichen darstellen "
  Public Shared ReadOnly Property ReturnEuroChar() As Char
    Get
      Return GetEuroChar()
    End Get
  End Property

  Private Shared Function GetEuroChar() As Char
    Return ChrW(&H20AC)
  End Function
#End Region
#End Region

#Region " Functions "
#Region " ... FormularOeffnen "
  Public Shared Function FormularOeffnen(ByVal mform As String) As Boolean
    ' ---------------------------------------------------------------------------------
    ' speichert Namen der Formulare
    ' ---------------------------------------------------------------------------------
    FormArray.Add(mform)
  End Function

  Public Shared Function FormularOffen(ByVal mform As String) As Boolean
    ' ---------------------------------------------------------------------------------
    ' gibt True zurück wenn Formular bereits geöffnet ist
    ' ---------------------------------------------------------------------------------
    Dim i As Int32 = -1

    FormArray.Sort() ' Array sortieren
    i = FormArray.IndexOf(mform) ' Suche ob namen vorhanden

    If Not i = -1 Then ' prüfen ob gefunden
      Return True
    Else
      Return False
    End If
  End Function
#End Region
#Region " ... FormSchliessen "
  Public Shared Function FormSchliessen(ByVal mform As String) As Boolean
    ' ---------------------------------------------------------------------------------
    ' löscht den Formularnamen
    ' ---------------------------------------------------------------------------------
    FormArray.Remove(mform)
  End Function
#End Region
#Region " ... GetRandom "
  Public Shared Function GetRandom(ByVal min As Integer, ByVal max As Integer) As Integer
    ' ---------------------------------------------------------------------------------
    '                               Zufallszahl, zwischen max- und min-Werten, erzeugen
    ' ---------------------------------------------------------------------------------
    Try
      Randomize() ' Initialisiert den Zufallszahlengenerator

      Dim iRnd As Integer

      Randomize()
      iRnd = CInt(min + (max - min + 1) * Rnd())

      While iRnd < min OrElse iRnd > max
        Randomize()
        iRnd = CInt(min + (max - min + 1) * Rnd())
      End While

      Return iRnd
    Catch ex As Exception
      ' ToDo Fehlerbehandlung
      Return min
    End Try
  End Function
#End Region
#Region " ... RoundCommercial "
  Public Shared Function RoundCommercial(ByVal value As Double, ByVal decimals As Integer) As Double
    ' ---------------------------------------------------------------------------------
    ' kaufmänisch Runden
    ' Aufruf z.B.: RoundCommercial(value, decimals)  
    ' ---------------------------------------------------------------------------------
    ' Vorzeichen merken
    Dim sign As Double = Math.Sign(value)

    ' Mit Absolutwert weiterrechnen
    value = Math.Abs(value)

    ' Zehnerpotenz zum Verschieben der Ziffern nach links
    Dim decFact As Double = Math.Pow(10, decimals)

    ' Alle bleibenden Stellen nach links schieben
    value *= decFact

    ' 0.5 addieren 
    value += 0.5

    ' Nachkommastellen abschneiden und durch Zehnerpotenz teilen
    Return sign * Math.Floor(value) / decFact
  End Function
#End Region
#Region " ... Dbl2Int "
  Public Shared Function Dbl2Int(ByVal value As Double) As Integer
    ' ---------------------------------------------------------------------------------
    ' Doublewert kaufmännisch, ohne Nachkommastelle, runden 
    ' und in einen Integerwert konvertieren 
    ' ---------------------------------------------------------------------------------
    Dim intx As Integer
    intx = CInt(RoundCommercial(value, 0))
    Return intx
  End Function
#End Region
#Region " ... GB Winkel-Umrechnung "
  Public Shared Function GB(ByVal value As Double) As Double
    ' ---------------------------------------------------------------------------------
    '                               Winkel-Umrechnung Gradmaß (°) in Bogenmaß (Radiant) 
    ' ---------------------------------------------------------------------------------
    Return value * Math.PI / 180
  End Function
#End Region
#Region " ... BG Winkel-Umrechnung "
  Public Shared Function BG(ByVal value As Double) As Double
    ' ---------------------------------------------------------------------------------
    '                               Winkel-Umrechnung Bogenmaß (Radiant) in Gradmaß (°)
    ' ---------------------------------------------------------------------------------
    Return value * 180 / Math.PI
  End Function
#End Region
#Region " ... Binary2Decimal "
  Public Shared Function Binary2Decimal(ByVal Bin As String) As Integer
    ' ---------------------------------------------------------------------------------
    ' Binärzahl nach Dezimal umwandeln                                Binary2Decimal(x)
    ' ---------------------------------------------------------------------------------
    ' z. B.: Binary2Decimal("11111111"), Ergebnis = 255
    Dim l As Integer

    For i As Integer = 0 To Bin.Length - 1
      l = (l << 1) + Integer.Parse(Bin.Substring(i, 1))
    Next
    Return l
  End Function
#End Region
#Region " ... Decimal2Binary "
  Public Shared Function Decimal2Binary(ByVal Dec As Integer) As String
    ' ---------------------------------------------------------------------------------
    ' Dezimal nach Binärzahl umwandeln                                Decimal2Binary(x)
    ' ---------------------------------------------------------------------------------
    Return Convert.ToString(Dec, 2)
  End Function
#End Region
#Region " ... Asin "
  Public Shared Function Asin(ByVal value As Double) As Double
    ' ---------------------------------------------------------------------------------
    ' Nachbildung von Asin(value)                                                Asin(x)
    ' Diese Funktion steht in VB nicht zur Verfügung
    ' ---------------------------------------------------------------------------------
    Return Math.Atan(value / Math.Sqrt(-value * value + 1))
  End Function
#End Region
#Region " ... Log10 "
  Public Shared Function Log10(ByVal x As Double) As Double
    ' ---------------------------------------------------------------------------------
    ' Nachbildung des dekadischen Logarithmus (zur Basis 10)                   Log10(x)
    ' Diese Funktion steht in VB nicht zur Verfügung
    ' ---------------------------------------------------------------------------------
    Return Math.Log(x) / Math.Log(10)
  End Function
#End Region
#Region " ... TestBit "
  Public Shared Function TestBit(ByVal value As Integer, ByVal bitNumber As Integer) As Boolean
    ' ---------------------------------------------------------------------------------
    ' Überprüfung, ob ein Bit in einem Integerwert gesetzt ist
    ' Funktion gibt True zurück, wenn Bit gesetzt ist.
    ' ---------------------------------------------------------------------------------
    ' Bitmaske für die Verknüpfung erzeugen
    Dim mask As Integer = 1 << bitNumber
    ' Testen, ob Bit gesetzt ist
    Return (value And mask) <> 0
  End Function
#End Region
#Region " ... BreakString "
  Public Shared Function BreakString(ByVal theString As String, ByVal approxLineWidth As Integer) As String
    ' ---------------------------------------------------------------------------------
    ' Lange Textzeilen umbrechen
    ' ---------------------------------------------------------------------------------
    Dim newString As String = ""
    Dim pos As Integer

    ' Umbrechen nur, solange der Text länger ist, als die 
    ' vorgegebene Breite
    Do While theString.Length > approxLineWidth

      ' Nächstes Leerzeichen suchen
      pos = theString.IndexOf(" "c, approxLineWidth)

      ' Wenn keins mehr gefunden wird, 
      If pos = -1 Then Return newString & theString

      ' Textfragment und Zeilenumbruch anhängen
      newString = newString & theString.Substring(0, pos) _
        & Environment.NewLine

      ' Rest weiterverarbeiten
      theString = theString.Substring(pos + 1)
    Loop
    Return newString & theString ' Text anhängen und zurückgeben
  End Function

  Public Shared Function BreakString(ByVal theString As String) As String
    ' ---------------------------------------------------------------------------------
    ' Lange Textzeilen umbrechen, 
    ' als überlagerte Methode, jedoch mit einer voreingestellten Umbruchsposition 
    ' ---------------------------------------------------------------------------------
    Return BreakString(theString, 24)
  End Function
#End Region
#Region " ... CheckSumme "
  Public Shared Function CheckSumme(ByVal strEingabe As String) As Integer
    ' ---------------------------------------------------------------------------------
    '                       Checksumme aus der Quersumme eines Eingabestrings berechnen
    ' ---------------------------------------------------------------------------------
    Dim n, intCSum As Integer

    If strEingabe.Length > 0 Then
      For n = 0 To strEingabe.Length - 1
        intCSum = intCSum + CInt(Asc(strEingabe.Chars(n)))
      Next
      intCSum = intCSum Mod 256
      Return intCSum
    Else
      Exit Function
    End If
  End Function
#End Region
#Region " ... IP-Adresse "
  ''' <summary>
  ''' Gibt einen String mit der ersten gefundenen gültigen IP zurück.
  ''' </summary>
  ''' <param name="GenerateIP">wenn keine IP gefunden wird, eine genereiren?</param>
  Public Shared Function GetLocalIP(Optional ByVal GenerateIP As Boolean = False) As String
    Try
      Dim IP As Net.IPAddress
      Dim IPList As Array = Net.Dns.GetHostEntry(SystemInformation.ComputerName.ToString).AddressList
      If IPList.Length > 0 Then
        For i As Integer = 0 To IPList.Length - 1
          IP = CType(IPList.GetValue(i), Net.IPAddress)
          If IP.ToString <> "0.0.0.0" Then Return IP.ToString
        Next
      End If
    Catch ex As Exception
    End Try

    If GenerateIP = True Then
      Return "192.168.1.1"
    Else
      Return "0.0.0.0"
    End If
  End Function
#End Region
#Region " ... Bytes darstellen "
  Public Shared Function ToByteString(ByVal bytes As Long) As String
    ' ---------------------------------------------------------------------------------
    ' Bytes in der höchsten Einheit darstellen    
    ' ---------------------------------------------------------------------------------
    Dim s As Long = bytes
    Dim format As String() = New String() {"{0} bytes", "{0} KB", "{0} MB", _
                                           "{0} GB", "{0} TB", "{0} PB", "{0} EB"}

    Dim i As Integer = 0

    While i < format.Length AndAlso s >= 1024
      s = CLng(CLng(100 * s \ 1024) / 100.0)
      i += 1
    End While
    Return String.Format(format(i), s)
  End Function
#End Region
#End Region

#Region " Procedures "
#Region " ... Delay "
  Public Shared Sub Delay(ByVal zeit As Integer)
    ' ---------------------------------------------------------------------------------
    ' Pause mit DoEvents-Schleife realisieren
    ' Aufruf z.B.: Delay(3000) entspricht 3 Sek.  
    ' ---------------------------------------------------------------------------------
    Dim zeitAlt As Integer
    zeitAlt = System.Environment.TickCount
    While (System.Environment.TickCount - zeitAlt) < zeit
      Application.DoEvents()
    End While
  End Sub
#End Region
#Region " ... Protokolldatei (LogFile.txt) schreiben"
  Public Shared Sub LogFile(ByVal Message As String)
    ' ---------------------------------------------------------------------------------
    '                                                Protokolldatei (LogFile) schreiben
    ' ---------------------------------------------------------------------------------
    Dim pfad As String = Environment.CurrentDirectory & "\LogFile.txt"
    Message = Date.Now & vbTab & Message

    ' Zuerst prüfen ob die Datei existiert
    If IO.File.Exists(pfad) = False Then
      ' Datei ist nicht vorhanden, somit neu anlegen und Message hineinschreiben
      Dim FiStr As FileStream = New FileStream(pfad, FileMode.Create)
      Dim StrWr As StreamWriter = New StreamWriter(FiStr)
      StrWr.WriteLine(Message)
      StrWr.Close()
    Else
      ' Message an bestehenden Text anfügen 
      Dim FiStr As FileStream = New FileStream(pfad, FileMode.Append) ' anfügen
      Dim StrWr As StreamWriter = New StreamWriter(FiStr)
      StrWr.WriteLine(Message)
      StrWr.Close()
    End If
  End Sub
#End Region
#End Region

End Class

#Region " Sortieren von ListView-Elementen "
' Sortieren von ListView-Elementen  *** Begin ***
' dieser Code stammt aus der .NET-CF-Hilfe von Microsoft
Public Class SortWrapper
  Friend sortItem As ListViewItem
  Friend sortColumn As Integer

  ' SortWrapper benötigt item und den index von der angeklickten Spalte
  Public Sub New(ByVal Item As ListViewItem, ByVal iColumn As Integer)
    sortItem = Item
    sortColumn = iColumn
  End Sub

  Public ReadOnly Property [Text]() As String
    Get
      Return sortItem.SubItems(sortColumn).Text
    End Get
  End Property

  ' Implementation of the IComparer 
  ' Interface zur Sortierung der ArrayList items.
  Public Class SortComparer
    Implements IComparer
    Private ascending As Boolean

    ' Constructor benötigt von der Sortierung
    ' true = ascending, False = descending
    Public Sub New(ByVal asc As Boolean)
      Me.ascending = asc
    End Sub

    Public Function [Compare](ByVal x As Object, ByVal y As Object) As Integer Implements IComparer.Compare
      ' ---------------------------------------------------------------------------------
      '                                                          zwei Objekte vergleichen
      ' ---------------------------------------------------------------------------------
      Dim xItem As SortWrapper = CType(x, SortWrapper)
      Dim yItem As SortWrapper = CType(y, SortWrapper)

      Dim xText As String = xItem.sortItem.SubItems(xItem.sortColumn).Text
      Dim yText As String = yItem.sortItem.SubItems(yItem.sortColumn).Text

      ' sollte auf- und absteigend sortiert werden, 
      ' so ist die nachstehende Zeile zu komplettieren
      ' Return xText.CompareTo(yText) '* CInt(IIf(Me.ascending, 1, -1)) nur aufsteigend
      Return xText.CompareTo(yText) * CInt(IIf(Me.ascending, 1, -1))
    End Function
  End Class
End Class

Public Class ColHeader
  Inherits ColumnHeader
  Public ascending As Boolean

  Public Sub New(ByVal [text] As String, ByVal width As Integer, ByVal align As HorizontalAlignment, ByVal asc As Boolean)
    Me.Text = [text]
    Me.Width = width
    Me.TextAlign = align
    Me.ascending = asc
  End Sub
End Class
#End Region